import io
import os
from setuptools import find_packages, setup

here = os.path.abspath(os.path.dirname(__file__))

with io.open(os.path.join(here, "README.rst"), "rt", encoding="utf8") as f:
    readme = f.read()

about = {}
with io.open(os.path.join(here, "grvlms", "__about__.py"), "rt", encoding="utf-8") as f:
    exec(f.read(), about)
requirements = []
with io.open(
    os.path.join(here, "requirements", "base.in"), "rt", encoding="utf-8"
) as f:
    requirements = [line.strip() for line in f]

setup(
    name="grvlms-openedx",
    version=about["__version__"],
    url="https://xyz",
    project_urls={
        "Documentation": "https://xyz",
        "Code": "https://xyz",
        "Issue tracker": "https://xyz",
        "Community": "https://xyz",
    },
    license="AGPLv3",
    author="GrooveTechnology",
    author_email="product@groovetechnology.com",
    description="The docker-based Open edX distribution designed for peace of mind",
    long_description=readme,
    long_description_content_type="text/x-rst",
    packages=find_packages(exclude=["tests*"]),
    include_package_data=True,
    python_requires=">=3.5",
    install_requires=requirements,
    entry_points={"console_scripts": ["grvlms=grvlms.commands.cli:main"]},
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU Affero General Public License v3",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
)
